/* Module      : JaegerGlobals.cpp
 * Author      : Robert W. Lindeman
 * Email       : gogo@wpi.edu
 * Course      : IMGD 3000
 *
 * Description : This file contains declarations for global variables.
 *
 * Date        : 2006/02/05
 *
 * History:
 * Revision      Date          Changed By
 * --------      ----------    ----------
 * 00.01         2006/02/05    gogo
 * First release.
 *
 * (c) Copyright 2006, Worcester Polytechnic Institute.
 */

/* -- INCLUDE FILES ------------------------------------------------------ */

#include <JaegerGlobals.h>
#include <JaegerCell.h>
#include <Camera.h>

/* -- DATA STRUCTURES ---------------------------------------------------- */

/* -- GLOBAL VARIABLES --------------------------------------------------- */

// Debug stuff.
bool JaegerGodMode = false; // Draw in the third person.

// A handle to the camera object.
Camera Cam;

// Screen dimensions.
int ScreenWidth = 800;
int ScreenHeight = 600;

// Viewing parameters.
float PlayerHeight = 180.0;         // Eye level for the player.
float NearClip    = 1.0;            // Near clipping plane.
float FarClip     = 10000.0;        // Far clipping plane.
float ViewAngle   = 90.0;           // Viewing angle.
float AspectRatio = 4.0 / 3.0;      // Aspect ration of the window.
Point3 EyePt( 0.0, PlayerHeight, 0.0 ); // Initial position of camera.
Point3 InterestPt( 500.0, PlayerHeight, 0.0 ); // Initial point of interest.
Vector3 UpVec( 0.0, 1.0, 0.0 );     // Initial "up" vector of camera.

// Window title.
char *WindowTitle = "Jaeger: The Hunt for Gert's Gold";

// The game maze.
Maze *GertMaze = NULL;

// The player.
Player *Hedi = NULL;

// Game State Variables.
Cell *CurCell = NULL;

// Cell attributes (units are in cm).
float CellSide      = 600;
float CellHeight    = 300;
float WallThickness = 15;


/* -- LOCAL VARIABLES ---------------------------------------------------- */

/* ----------------------------------------------------------------------- */
