/* Module      : JaegerMain.cpp
 * Author      : Robert W. Lindeman
 * Email       : gogo@wpi.edu
 * Course      : IMGD 3000
 *
 * Description : This is the main module of the Jaeger game application.
 *               Not much happens here, as each major subsystem has its
 *               own module, so that it can be removed and rewritten as
 *               part of class assignments.
 *
 * Date        : 2006/02/05
 *
 * History:
 * Revision      Date          Changed By
 * --------      ----------    ----------
 * 00.01         2006/02/05    gogo
 * First release.
 *
 * (c) Copyright 2006, Worcester Polytechnic Institute.
 */

/* -- INCLUDE FILES ------------------------------------------------------ */

#include <stdlib.h>
#include <string.h>

#ifdef MACOSX
  #include <gl.h>
  #include <glu.h>
  #include <glut.h>
#else
  #include <GL/gl.h>
  #include <GL/glu.h>
  #include <GL/glut.h>
#endif

#include <JaegerDraw.h>
#include <JaegerMaze.h>
#include <JaegerGlobals.h>

/* -- DATA STRUCTURES ---------------------------------------------------- */

/* -- GLOBAL VARIABLES --------------------------------------------------- */

/* -- LOCAL VARIABLES ---------------------------------------------------- */

// Sizes to make the labyrinth.
static int MWidth = 10;
static int MHeight = 10;

// Flag: Should we generate a default maze?
static bool GenerateRandomMaze = false;

// Seed to use for random-number generation.
static float RandomSeed = 0.0;

// File names of files to load from / save to (filled from the command line).
static char LoadFile[257];
static char SaveFile[257];

/* ----------------------------------------------------------------------- */
/* Function    : void JaegerInit( void )
 *
 * Description : This function is used to initialize the game state.
 *
 * Parameters  : void
 *
 * Returns     : void
 */

void JaegerInit( void )  {
  ;
}

/* ----------------------------------------------------------------------- */
/* Function    : void JaegerKeyCommands( void )
 *
 * Description : This function prints a list of keys that can be used in
 *               the game.
 *
 * Parameters  : void
 *
 * Returns     : void
 */

void JaegerKeyCommands( void )  {
  printf( "Keyboard Commands:\n" );
  /*
  printf( "  r      : Throw a RED Jellybean.\n" );
  printf( "  g      : Throw a GREEN Jellybean.\n" );
  printf( "  b: Throw a BLUE Jellybean.\n" );
  */
  printf( "  Left Arrow : Turn 90 deg. to the left.\n" );
  printf( "  Right Arrow: Turn 90 deg. to the right.\n" );
  printf( "  Up Arrow   : Move forward one cell.\n" );
  printf( "  Down Arrow : Move backward one cell.\n" );

  printf( "  s          : Save the labyrinth to a file.\n" );

  printf( "  ESC        : Exit.\n" );
  printf( "  q          : Exit.\n" );
}

/* ----------------------------------------------------------------------- */
/* Function    : void JaegerKeyboard( unsigned char k, int x, int y )
 *
 * Description : This function handles key presses for "standard" keyboard
 *               keys.
 *
 * Parameters  : unsigned char k : Key pressed by the user. 
 *               int x           : X coordinate of the mouse when the key
 *                                 was pressed.
 *               int y           : Y coordinate of the mouse when the key
 *                                 was pressed.
 *
 * Returns     : void
 */

void JaegerKeyboard( unsigned char k, int x, int y )  {
  float SlideUnit = 25;

  switch( k )  {
    case 'r': // Throw a RED jellybean.
      break;
    case 'g': // Throw a GREEN jellybean.
      break;
    case 'b': // Throw a BLUE jellybean.
      break;

    case 'o': // Move back.
      Cam.Slide( 0.0, 0.0, SlideUnit );
      break;
    case 'u': // Move forward.
      Cam.Slide( 0.0, 0.0, -SlideUnit );
      break;
    case 'i': // Move up.
      Cam.Slide( 0.0, SlideUnit, 0.0 );
      break;
    case 'k': // Move down.
      Cam.Slide( 0.0, -SlideUnit, 0.0 );
      break;
    case 'l': // Move right.
      Cam.Slide( SlideUnit, 0.0, 0.0 );
      break;
    case 'j': // Move left.
      Cam.Slide( -SlideUnit, 0.0, 0.0 );
      break;
    case 'n': // Turn left
      Cam.Yaw( 5.0 );
      break;
    case 'm': // Turn right
      Cam.Yaw( -5.0 );
      break;
    case 'C': // Roll right.
      Cam.Roll( 5.0 );
      break;
    case 'Z': // Roll left.
      Cam.Roll( -5.0 );
      break;
    case ',': // Turn up
      Cam.Pitch( 5.0 );
      break;
    case '.': // Turn down
      Cam.Pitch( -5.0 );
      break;

    case 's': // Save the labyrinth to a file.
      GertMaze->Save( SaveFile );
      break;

    case 'q': // Exit the game.
    case 27:  // Escape key does the same.
      exit( 0 );
      break;
  }

  glutPostRedisplay( );
}

/* ----------------------------------------------------------------------- */
/* Function    : void JaegerKeyboardSpecial( int k, int x, int y )
 *
 * Description : This function handles key presses for "special" keyboard
 *               keys, like the cursor keys.
 *
 * Parameters  : int k : Key pressed by the user. 
 *               int x : X coordinate of the mouse when the key
 *                       was pressed.
 *               int y : Y coordinate of the mouse when the key
 *                       was pressed.
 *
 * Returns     : void
 */

void JaegerKeyboardSpecial( int k, int x, int y )  {
  int mod = glutGetModifiers( ); // Get the state of SHIFT, CTRL, etc.

  if( mod & GLUT_ACTIVE_CTRL )  {
    printf( "Ctrl Held " );
  }
  if( mod & GLUT_ACTIVE_SHIFT )  {
    printf( "Shift Held " );
  }
  if( mod & GLUT_ACTIVE_ALT )  {
    printf( "Alt Held " );
  }

  // based on the key, update the camera.
  switch( k )  {
    case GLUT_KEY_UP: // Up-arrow key.
      JaegerMoveForwardCell( );
      break;
    case GLUT_KEY_DOWN: // Down-arrow key.
      JaegerMoveBackCell( );
      break;
    case GLUT_KEY_LEFT: // Left-arrow key.
      JaegerTurnLeft90( );
      break;
    case GLUT_KEY_RIGHT: // Right-arrow key.
      JaegerTurnRight90( );
      break;
    default:
      printf( "Special key (%d) not implemented yet!\n", k );
      break;
  }
  printf( "Hedi is in cell ( %d, %d )", Hedi->pos[Xidx], Hedi->pos[Yidx] );
  printf( " and facing: %d\n", Hedi->ori );

  glutPostRedisplay( );
}

/* ----------------------------------------------------------------------- */
/* Function    : void JaegerUsage( int argc, char *argv[] )
 *
 * Description : This function prints a usage message.
 *
 * Parameters  : int argc     : Number of command-line arguments.
 *               char *argv[] : Array of command-line arguments.
 *
 * Returns     : void
 */

void JaegerUsage( int argc, char *argv[] )  {
  printf( "use: %s [options]*\n\n", &argv[0][0] );
  printf( "options:\n" );
  printf( "  -G       : Turn 'God' mode on.\n" );
  printf( "  -hHeight : Use 'Height' as the labyrinth height.\n" );
  printf( "  -lFile   : Load the labyrinth from \'File\'.\n" );
  printf( "  -r       : Generate a random labyrinth.\n" );
  printf( "  -sSeed   : Use 'Seed' for random-number generation.\n" );
  printf( "  -SFile   : When told to, save the current labyrinth to \'File\'.\n" );
  printf( "             Default file is 'savemaze.txt'.\n" );
  printf( "  -wWidth  : Use 'Width' as the labyrinth width.\n" );
  printf( "  -?       : Print this help message.\n\n" );
}

/* ----------------------------------------------------------------------- */
/* Function    : int JaegerProcessCommandLine( int argc, char *argv[] )
 *
 * Description : This function processes the command-line arguments.
 *
 * Parameters  : int argc     : Number of command-line arguments.
 *               char *argv[] : Array of command-line arguments.
 *
 * Returns     : int :  0 = OK.
 *                     -1 = Error, or usage message printed.
 */

int JaegerProcessCommandLine( int argc, char *argv[] )  {
  int retVal = 0;
  // Process command-line parameters.
  for( int i = 1; i < argc; i++ )  {
    if( argv[i][0] == '-' )  {
      switch( argv[i][1] )  {
        case 'G': // Turn 'God' mode on.
          JaegerGodMode = true;
          break;
        case 'h': // Use this as the labyrinth height.
          MHeight = atoi( &argv[i][2] );
          break;
        case 'l': // Load the maze from a file.
          strcpy( LoadFile, &argv[i][2] );
          break;
        case 'r': // Generate a random maze.
          GenerateRandomMaze = true;
          break;
        case 's': // Use this seed for random-number generation.
          RandomSeed = atof( &argv[i][2] );
          break;
        case 'S': // Save the maze to a file.
          strcpy( SaveFile, &argv[i][2] );
          break;
        case 'w': // Use this as the labyrinth width.
          MWidth = atoi( &argv[i][2] );
          break;
        case '?': // Output usage message, and exit.
          JaegerUsage( argc, argv );
          retVal = -1;
          break;
        default:
          fprintf( stderr, "Unknown option: \"%s\"\n\n", &argv[i][0] ); 
          JaegerUsage( argc, argv );
          retVal = -1;
          break;
      }
    }
  }
  return( retVal );
}

/* ----------------------------------------------------------------------- */
/* Function    : int main( int argc, char *argv[] )
 *
 * Description : This is the main function of Jaeger. It sets things up,
 *               and then enters the rendering/event loop.
 *
 * Parameters  : int argc     : Number of command-line arguments.
 *               char *argv[] : Array of command-line arguments.
 *
 * Returns     : int :  0 = OK.
 *                     -1 = Error, or usage message printed.
 */

int main( int argc, char** argv )  {
  // Set the default file names.
  strcpy( LoadFile, "" );
  strcpy( SaveFile, "savemaze.txt" );

  // Process command-line parameters.
  if( JaegerProcessCommandLine( argc, argv ) != 0 )  {
    return( -1 );
  }

  // Initialize graphics stuff.
  JaegerDrawInit( argc, argv );
  
  // Set callback functions for events.
  glutKeyboardFunc( JaegerKeyboard );
  glutSpecialFunc( JaegerKeyboardSpecial );
  
  // Initialize game values.
  JaegerInit( );

  // Output a list of commands the user can use.
  JaegerKeyCommands( );
  
  if( strlen( LoadFile ) == 0 )  {
    // Create the labyrinth.
    GertMaze = new Maze( MWidth, MHeight );  // Allocate the space.
    // Now fill it.
    if( GenerateRandomMaze )  {
      GertMaze->GenerateRandomMaze( RandomSeed ); // Generate a random one.
    }
    else  {
      GertMaze->GenerateDefaultMaze( RandomSeed ); // Use the default.
    }
  } else  {
    // Create the labyrinth.
    GertMaze = new Maze( );
    GertMaze->Load( LoadFile );
  }
  GertMaze->Print( ); // Print it in ASCII, just to check.

  // Make Hedi, and place him in the game.
  Hedi = new Player( 0, 0 );
  Hedi->ori = ORI_EAST;

  // Enter the event loop.
  JaegerMainLoop( );

  return( 0 );
}

/* ----------------------------------------------------------------------- */
